package com.jeeplus.modules.test.project.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.test.note.entity.TestNote;
import com.jeeplus.modules.test.note.service.TestNoteService;
import com.jeeplus.modules.test.project.entity.ProjectMaintenence;
import com.jeeplus.modules.test.project.service.ProjectMaintenenceService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;
import java.util.Map;

/**
 *
 */
@Controller
@RequestMapping(value = "${adminPath}/test/project/maintenence")
public class ProjectMaintenenceController extends BaseController {

    @Autowired
    private ProjectMaintenenceService projectMaintenenceService;

    @ModelAttribute
    public ProjectMaintenence get(@RequestParam(required=false) String id) {
        ProjectMaintenence entity = null;
        if (StringUtils.isNotBlank(id)){
            entity = projectMaintenenceService.get(id);
        }
        if (entity == null){
            entity = new ProjectMaintenence();
        }
        return entity;
    }

    /**
     *
     */
    @RequiresPermissions("test:project:maintenence:list")
    @RequestMapping(value = {"list", ""})
    public String list(ProjectMaintenence projectMaintenence, Model model) {
        model.addAttribute("projectMaintenence", projectMaintenence);
        return "modules/test/project/projectMaintenenceList";
    }

    /**
     *
     */
    @ResponseBody
    @RequiresPermissions("test:project:maintenence:list")
    @RequestMapping(value = "data")
    public Map<String, Object> data(ProjectMaintenence testNote, HttpServletRequest request, HttpServletResponse response, Model model) {
        Page<ProjectMaintenence> page = projectMaintenenceService.findPage(new Page<ProjectMaintenence>(request, response), testNote);
        return getBootstrapData(page);
    }

    /**
     *
     */
    @RequiresPermissions(value={"test:project:maintenence:view","test:project:maintenence:add","test:project:maintenence:edit"},logical=Logical.OR)
    @RequestMapping(value = "form/{mode}")
    public String form(@PathVariable String mode, ProjectMaintenence projectMaintenence, Model model) {
        model.addAttribute("projectMaintenence", projectMaintenence);
        model.addAttribute("mode", mode);
        return "modules/test/project/projectMaintenenceForm";
    }

    /**
     *
     */
    @ResponseBody
    @RequiresPermissions(value={"test:project:maintenence:add","test:project:maintenence:edit"},logical=Logical.OR)
    @RequestMapping(value = "save")
    public AjaxJson save(ProjectMaintenence projectMaintenence, Model model) throws Exception{
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(projectMaintenence);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        projectMaintenenceService.save(projectMaintenence);//保存
        j.setSuccess(true);
        j.setMsg("保存项目运营管理成功");
        return j;
    }

    /**
     *
     */
    @ResponseBody
    @RequiresPermissions("test:project:maintenence:del")
    @RequestMapping(value = "delete")
    public AjaxJson delete(ProjectMaintenence projectMaintenence) {
        AjaxJson j = new AjaxJson();
        projectMaintenenceService.delete(projectMaintenence);
        j.setMsg("删除项目运营管理成功");
        return j;
    }

    /**
     *
     */
    @ResponseBody
    @RequiresPermissions("test:project:maintenence:del")
    @RequestMapping(value = "deleteAll")
    public AjaxJson deleteAll(String ids) {
        AjaxJson j = new AjaxJson();
        String idArray[] =ids.split(",");
        for(String id : idArray){
            projectMaintenenceService.delete(projectMaintenenceService.get(id));
        }
        j.setMsg("删除项目运营管理成功");
        return j;
    }

    /**
     *
     */
    @ResponseBody
    @RequiresPermissions("test:project:maintenence:export")
    @RequestMapping(value = "export")
    public AjaxJson exportFile(ProjectMaintenence projectMaintenence, HttpServletRequest request, HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "项目运营管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ProjectMaintenence> page = projectMaintenenceService.findPage(new Page<ProjectMaintenence>(request, response, -1), projectMaintenence);
            new ExportExcel("项目运营管理", ProjectMaintenence.class).setDataList(page.getList()).write(response, fileName).dispose();
            j.setSuccess(true);
            j.setMsg("导出成功！");
            return j;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导出项目运营管理记录失败！失败信息："+e.getMessage());
        }
        return j;
    }

    /**
     *

     */
    @ResponseBody
    @RequiresPermissions("test:project:maintenence:import")
    @RequestMapping(value = "import")
    public AjaxJson importFile(@RequestParam("file")MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<ProjectMaintenence> list = ei.getDataList(ProjectMaintenence.class);
            for (ProjectMaintenence projectMaintenence : list){
                try{
                    projectMaintenenceService.save(projectMaintenence);
                    successNum++;
                }catch(ConstraintViolationException ex){
                    failureNum++;
                }catch (Exception ex) {
                    failureNum++;
                }
            }
            if (failureNum>0){
                failureMsg.insert(0, "，失败 "+failureNum+" 条项目运营管理记录。");
            }
            j.setMsg( "已成功导入 "+successNum+" 条项目运营管理记录"+failureMsg);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目运营管理失败！失败信息："+e.getMessage());
        }
        return j;
    }

    /**
     *
     */
    @ResponseBody
    @RequiresPermissions("test:project:maintenence:import")
    @RequestMapping(value = "import/template")
    public AjaxJson importFileTemplate(HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "项目运营管理数据导入模板.xlsx";
            List<ProjectMaintenence> list = Lists.newArrayList();
            new ExportExcel("项目运营管理数据", ProjectMaintenence.class, 1).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg( "导入模板下载失败！失败信息："+e.getMessage());
        }
        return j;
    }

}